From: tip-bot for Michal Hocko <mhocko@suse.cz>
To: linux-tip-commits@vger.kernel.org
Cc: linux-kernel@vger.kernel.org, hpa@zytor.com, mingo@redhat.com,
stable@kernel.org, jkosina@suse.cz, tglx@linutronix.de,
mhocko@suse.cz
Subject: [tip:x86/urgent] x86: Increase MIN_GAP to include randomized stack
Date: Fri, 11 Sep 2009 00:06:03 GMT [thread overview]
Message-ID: <tip-80938332d8cf652f6b16e0788cf0ca136befe0b5@git.kernel.org> (raw)
In-Reply-To: <1252400515-6866-1-git-send-email-mhocko@suse.cz>
Commit-ID: 80938332d8cf652f6b16e0788cf0ca136befe0b5
Gitweb: http://git.kernel.org/tip/80938332d8cf652f6b16e0788cf0ca136befe0b5
Author: Michal Hocko <mhocko@suse.cz>
AuthorDate: Tue, 8 Sep 2009 11:01:55 +0200
Committer: H. Peter Anvin <hpa@zytor.com>
CommitDate: Thu, 10 Sep 2009 17:00:12 -0700
x86: Increase MIN_GAP to include randomized stack
Currently we are not including randomized stack size when calculating
mmap_base address in arch_pick_mmap_layout for topdown case. This might
cause that mmap_base starts in the stack reserved area because stack is
randomized by 1GB for 64b (8MB for 32b) and the minimum gap is 128MB.
If the stack really grows down to mmap_base then we can get silent mmap
region overwrite by the stack values.
Let's include maximum stack randomization size into MIN_GAP which is
used as the low bound for the gap in mmap.
Signed-off-by: Michal Hocko <mhocko@suse.cz>
LKML-Reference: <1252400515-6866-1-git-send-email-mhocko@suse.cz>
Acked-by: Jiri Kosina <jkosina@suse.cz>
Signed-off-by: H. Peter Anvin <hpa@zytor.com>
Cc: Stable Team <stable@kernel.org>
---
arch/x86/include/asm/elf.h | 2 ++
arch/x86/mm/mmap.c | 17 +++++++++++++++--
2 files changed, 17 insertions(+), 2 deletions(-)
diff --git a/arch/x86/include/asm/elf.h b/arch/x86/include/asm/elf.h
index 83c1bc8..456a304 100644
--- a/arch/x86/include/asm/elf.h
+++ b/arch/x86/include/asm/elf.h
@@ -299,6 +299,8 @@ do { \
#ifdef CONFIG_X86_32
+#define STACK_RND_MASK (0x7ff)
+
#define VDSO_HIGH_BASE (__fix_to_virt(FIX_VDSO))
#define ARCH_DLINFO ARCH_DLINFO_IA32(vdso_enabled)
diff --git a/arch/x86/mm/mmap.c b/arch/x86/mm/mmap.c
index 1658296..c8191de 100644
--- a/arch/x86/mm/mmap.c
+++ b/arch/x86/mm/mmap.c
@@ -29,13 +29,26 @@
#include <linux/random.h>
#include <linux/limits.h>
#include <linux/sched.h>
+#include <asm/elf.h>
+
+static unsigned int stack_maxrandom_size(void)
+{
+ unsigned int max = 0;
+ if ((current->flags & PF_RANDOMIZE) &&
+ !(current->personality & ADDR_NO_RANDOMIZE)) {
+ max = ((-1U) & STACK_RND_MASK) << PAGE_SHIFT;
+ }
+
+ return max;
+}
+
/*
* Top of mmap area (just below the process stack).
*
* Leave an at least ~128 MB hole.
*/
-#define MIN_GAP (128*1024*1024)
+#define MIN_GAP (128*1024*1024UL + stack_maxrandom_size())
#define MAX_GAP (TASK_SIZE/6*5)
/*
prev parent reply other threads:[~2009-09-11 0:06 UTC|newest]
Thread overview: 12+ messages / expand[flat|nested] mbox.gz Atom feed top
2009-09-04 9:37 [PATCH v2] x86: increase MIN_GAP to include randomized stack Michal Hocko
2009-09-07 8:28 ` Michal Hocko
2009-09-07 15:18 ` Jiri Kosina
2009-09-08 7:32 ` Michal Hocko
2009-09-08 8:43 ` [PATCH v3] " Michal Hocko
2009-09-08 8:47 ` Jiri Kosina
2009-09-08 8:53 ` Michal Hocko
2009-09-08 9:01 ` [PATCH v4] " Michal Hocko
2009-09-08 9:09 ` Jiri Kosina
2009-09-10 23:14 ` Jiri Kosina
2009-09-10 23:29 ` H. Peter Anvin
2009-09-11 0:06 ` tip-bot for Michal Hocko [this message]
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=tip-80938332d8cf652f6b16e0788cf0ca136befe0b5@git.kernel.org \
--to=mhocko@suse.cz \
--cc=hpa@zytor.com \
--cc=jkosina@suse.cz \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-tip-commits@vger.kernel.org \
--cc=mingo@redhat.com \
--cc=stable@kernel.org \
--cc=tglx@linutronix.de \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -29,13 +29,26 @@
* Top of mmap area (just below the process stack).
*
- * Leave an at least ~128 MB hole.
+ * Leave an at least ~128 MB hole with possible stack randomization.
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.